/*
 * Copyright(C) OASIS(R) 2005,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 */
package org.oasisopen.sdo;

import java.io.Serializable;
import java.util.List;

/**
 * A data object is a representation of some structured data.
 * It is the fundamental component in the SDO (Service Data Objects) package.
 * Data objects support reflection, path-based access, convenience creation and deletion methods,
 * and the ability to be part of a data graph.
 * <p>
 * Each data object holds its data as a series of {@link Property Properties}.
 * Properties can be accessed by name, property index, or using the property meta object itself.
 * A data object can also contain references to other data objects, through reference-type Properties.
 * <p>
 * A data object has a series of convenience accessors for its Properties.
 * These methods either use a path (String),
 * a property index,
 * or the {@link Property property's meta object} itself, to identify the property.
 * Some examples of the path-based accessors are as follows:
 *<pre>
 * DataObject company = ...;
 * company.get("name");                   is the same as company.get(company.getType().getProperty("name"))
 * company.set("name", "acme");
 * company.get("department.0/name")       is the same as ((DataObject)((List)company.get("department")).get(0)).get("name")
 *                                        .n  indexes from 0 ... implies the name property of the first department
 * company.get("department[1]/name")      [] indexes from 1 ... implies the name property of the first department
 * company.get("department[number=123]")  returns the first department where number=123
 * company.get("..")                      returns the containing data object
 * company.get("/")                       returns the root containing data object
 *</pre>
 * <p> There are general accessors for Properties, i.e., {@link #get(Property) get} and {@link #set(Property, Object) set},
 * as well as specific accessors for the primitive types and commonly used data types like
 * String, Date, List, BigInteger, and BigDecimal.
 */
public interface DataObject extends Serializable
{

  /**
   * Returns the value of a property of either this object or an object reachable from it, as identified by the
   * specified path.
   * @param path the path to a valid object and property.
   * @return the value of the specified property.
   * @see #get(Property)
   */
  Object get(String path);

  /**
   * Sets a property of either this object or an object reachable from it, as identified by the specified path,
   * to the specified value.
   * @param path the path to a valid object and property.
   * @param value the new value for the property.
   * @see #set(Property, Object)
   */
  void set(String path, Object value);

  /**
   * Returns whether a property of either this object or an object reachable from it, as identified by the specified path,
   * is considered to be set.
   * @param path the path to a valid object and property.
   * @see #isSet(Property)
   */
  boolean isSet(String path);

  /**
   * Unsets a property of either this object or an object reachable from it, as identified by the specified path.
   * @param path the path to a valid object and property.
   * @see #unset(Property)
   */
  void unset(String path);

  /**
   * Sets the value of a <code>List</code> property identified by the specified path, to the specified value.
   * @param path the path to a valid object and property.
   * @param value the new value for the property.
   * @see #set(String, Object)
   * @see #setList(Property, List)
   */
  void setList(String path, List<? extends Object> value);

  /**
   * Returns the value of the property at the specified index in {@link Type#getProperties property list}
   * of this object's {@link Type type}.
   * @param propertyIndex the index of the property.
   * @return the value of the specified property.
   * @see #get(Property)
   */
  Object get(int propertyIndex);

  /**
   * Sets the property at the specified index in {@link Type#getProperties property list} of this object's
   * {@link Type type}, to the specified value.
   * @param propertyIndex the index of the property.
   * @param value the new value for the property.
   * @see #set(Property, Object)
   */
  void set(int propertyIndex, Object value);

  /**
   * Converts the provided value, and uses it to set the value of the property identified by the specified path.
   * @param targetClass if the conversion is supported, the value will be converted
   * from this class.
   * @param path the path to a valid object and property.
   * @param value the new value for the property.
   * @see #set(String, Object)
   */
  <T> void set(Class<T> sourceClass, String path, T value);

  /**
   * Converts the provided value, and uses it to set the value of the property identified by the specified property index.
   * @param targetClass if the conversion is supported, the value will be converted
   * from this class.
   * @param propertyIndex the index of the property.
   * @param value the new value for the property.
   * @see #set(int, Object)
   */
  <T> void set(Class<T> sourceClass, int propertyIndex, T value);

  /**
   * Converts the provided value, and uses it to set the value of the property.
   * @param targetClass if the conversion is supported, the value will be converted
   * from this class.
   * @param property the property to set.
   * @param value the new value for the property.
   * @see #set(Property, Object)
   */
  <T> void set(Class<T> sourceClass, Property property, T value);

  /**
   * Returns whether the the property at the specified index in {@link Type#getProperties property list} of this object's
   * {@link Type type}, is considered to be set.
   * @param propertyIndex the index of the property.
   * @return whether the specified property is set.
   * @see #isSet(Property)
   */
  boolean isSet(int propertyIndex);

  /**
   * Unsets the property at the specified index in {@link Type#getProperties property list} of this object's {@link Type type}.
   * @param propertyIndex the index of the property.
   * @see #unset(Property)
   */
  void unset(int propertyIndex);

  /**
   * Returns the value of the given property of this object.
   * <p>
   * If the property is {@link Property#isMany many-valued},
   * the result will be a {@link java.util.List}
   * and each object in the List will be {@link Type#isInstance an instance of}
   * the property's {@link Property#getType type}.
   * Otherwise the result will directly be an instance of the property's type.
   * @param property the property of the value to fetch.
   * @return the value of the given property of the object.
   * @see #set(Property, Object)
   * @see #unset(Property)
   * @see #isSet(Property)
   */
  Object get(Property property);

  /**
   * Converts the value of the property identified by the specified path to the provided
   * target class.
   * @param targetClass if the conversion is supported, the property value will be converted
   * to this class.
   * @param path the path to a valid object and property.
   * @return the converted value of the specified property.
   * @see #get(String)
   */
  <T> T get(Class<T> targetClass, String path);

  /**
   * Converts the value of the property identified by the specified property index to the provided
   * target class.
   * @param targetClass if the conversion is supported, the property value will be converted
   * to this class.
   * @param propertyIndex the index of the property.
   * @return the converted value of the specified property.
   * @see #get(int)
   */
  <T> T get(Class<T> targetClass, int propertyIndex);

  /**
   * Converts the value of the specified property to the provided target class.
   * @param targetClass if the conversion is supported, the property value will be converted
   * to this class.
   * @param property the property to get.
   * @return the converted value of the specified property.
   * @see #get(Property)
   */
  <T> T get(Class<T> targetClass, Property property);

  /**
   * Returns the value of the property identified by the specified path as the specified type of list.
   * @param targetClass if the conversion is supported, the elements of the list will be converted
   * to this class.
   * @param path the path to a valid object and property.
   * @return the <code>List</code> value of the specified property.
   * @see #get(String)
   */
  <T> List<T> getList(Class<T> targetClass, String path);

  /**
   * Returns the value of the property identified by the specified property index as the specified type of list.
   * @param targetClass if the conversion is supported, the elements of the list will be converted
   * to this class.
   * @param propertyIndex the index of the property.
   * @return the <code>List</code> value of the specified property.
   * @see #get(int)
   */
  <T> List<T> getList(Class<T> targetClass, int propertyIndex);

  /**
   * Returns the value of the property.
   * Updates through the List interface operate on the current values of the DataObject.
   * @param targetClass if the conversion is supported, the elements of the list will be converted
   * to this class.
   * @param property the property to get.
   * @return the <code>List</code> value of the specified property.
   * @see #get(Property)
   */
  <T> List<T> getList(Class<T> targetClass, Property property);

  /**
   * Sets the value of the given property of the object to the new value.
   * <p>
   * If the property is {@link Property#isMany many-valued},
   * the new value must be a {@link java.util.List}
   * and each object in that list must be {@link Type#isInstance an instance of}
   * the property's {@link Property#getType type};
   * the existing contents are cleared and the contents of the new value are added.
   * Otherwise the new value directly must be an instance of the property's type
   * and it becomes the new value of the property of the object.
   * @param property the property of the value to set.
   * @param value the new value for the property.
   * @see #unset(Property)
   * @see #isSet(Property)
   * @see #get(Property)
   */
  void set(Property property, Object value);

  /**
   * Returns whether the property of the object is considered to be set.
   * <p>
   * isSet() for many-valued Properties returns true if the List is not empty and
   * false if the List is empty. For single-valued Properties it returns true if the Property
   * has been set() and not unset(), and false otherwise.
   * Any call to set() without a call to unset() will cause isSet() to return true, regardless of
   * the value being set. For example, after calling set(property, property.getDefault()) on a
   * previously unset property, isSet(property) will return true, even though the value of
   * get(property) will be unchanged.
   * @param property the property in question.
   * @return whether the property of the object is set.
   * @see #set(Property, Object)
   * @see #unset(Property)
   * @see #get(Property)
   */
  boolean isSet(Property property);

  /**
   * Unsets the property of the object.
   * <p>
   * If the property is {@link Property#isMany many-valued},
   * the value must be an {@link java.util.List}
   * and that list is cleared.
   * Otherwise,
   * the value of the property of the object
   * is set to the property's {@link Property#getDefault default value}.
   * The property will no longer be considered {@link #isSet set}.
   * @param property the property in question.
   * @see #isSet(Property)
   * @see #set(Property, Object)
   * @see #get(Property)
   */
  void unset(Property property);

  /**
   * Sets the value of the specified <code>List</code> property, to the specified value.
   * <p> The new value must be a {@link java.util.List}
   * and each object in that list must be {@link Type#isInstance an instance of}
   * the property's {@link Property#getType type};
   * the existing contents are cleared and the contents of the new value are added.
   * @param property the property to set.
   * @param value the new value for the property.
   * @see #set(Property, Object)
   */
  void setList(Property property, List<? extends Object> value);

  /**
   * Returns a new {@link DataObject data object} contained by this object using the specified property,
   * which must be a {@link Property#isContainment containment property}.
   * The type of the created object is the {@link Property#getType declared type} of the specified property.
   * @param propertyName the name of the specified containment property.
   * @return the created data object.
   * @see #createDataObject(String, String, String)
   */
  DataObject createDataObject(String propertyName);

  /**
   * Returns a new {@link DataObject data object} contained by this object using the specified property,
   * which must be a {@link Property#isContainment containment property}.
   * The type of the created object is the {@link Property#getType declared type} of the specified property.
   * @param propertyIndex the index of the specified containment property.
   * @return the created data object.
   * @see #createDataObject(int, String, String)
   */
  DataObject createDataObject(int propertyIndex);

  /**
   * Returns a new {@link DataObject data object} contained by this object using the specified property,
   * which must be a {@link Property#isContainment containment property}.
   * The type of the created object is the {@link Property#getType declared type} of the specified property.
   * @param property the specified containment property.
   * @return the created data object.
   * @see #createDataObject(Property, Type)
   */
  DataObject createDataObject(Property property);

  /**
   * Returns a new {@link DataObject data object} contained by this object using the specified property,
   * which must be a {@link Property#isContainment containment property}.
   * The type of the created object is specified by the packageURI and typeName arguments.
   * The specified type must be a compatible target for the property identified by propertyName.
   * @param propertyName the name of the specified containment property.
   * @param namespaceURI the namespace URI of the package containing the type of object to be created.
   * @param typeName the name of a type in the specified package.
   * @return the created data object.
   * @see #createDataObject(String)
   * @see org.oasisopen.sdo.helper.TypeHelper#getType
   */
  DataObject createDataObject(String propertyName, String namespaceURI, String typeName);

  /**
   * Returns a new {@link DataObject data object} contained by this object using the specified property,
   * which must be a {@link Property#isContainment containment property}.
   * The type of the created object is specified by the packageURI and typeName arguments.
   * The specified type must be a compatible target for the property identified by propertyIndex.
   * @param propertyIndex the index of the specified containment property.
   * @param namespaceURI the namespace URI of the package containing the type of object to be created.
   * @param typeName the name of a type in the specified package.
   * @return the created data object.
   * @see #createDataObject(int)
   * @see org.oasisopen.sdo.helper.TypeHelper#getType
   */
  DataObject createDataObject(int propertyIndex, String namespaceURI, String typeName);

  /**
   * Returns a new {@link DataObject data object} contained by this object using the specified property,
   * which must be of {@link Property#isContainment containment type}.
   * The type of the created object is specified by the type argument,
   * which must be a compatible target for the specified property.
   * @param property a containment property of this object.
   * @param type the type of object to be created.
   * @return the created data object.
   * @see #createDataObject(int)
   */
  DataObject createDataObject(Property property, Type type);

  /**
   * Remove this object from its container and then unset all its non-{@link Property#isReadOnly readOnly} Properties.
   * If this object is contained by a {@link Property#isReadOnly readOnly} {@link Property#isContainment containment property}, its non-{@link Property#isReadOnly readOnly} Properties will be unset but the object will not be removed from its container.
   * All DataObjects recursively contained by {@link Property#isContainment containment Properties} will also be deleted.
   */
  void delete();

  /**
   * Returns the containing {@link DataObject data object}
   * or <code>null</code> if there is no container.
   * @return the containing data object or <code>null</code>.
   */
  DataObject getContainer();

  /**
   * Return the Property of the {@link DataObject data object} containing this data object
   * or <code>null</code> if there is no container.
   * @return the property containing this data object.
   */
  Property getContainmentProperty();

  /**
   * Returns the data object's type.
   * <p>
   * The type defines the Properties available for reflective access.
   * @return the type.
   */
  Type getType();

  /**
   * Returns a read-only List of the Properties currently used in this DataObject.
   * This list will contain all of the Properties in getType().getProperties()
   * and any Properties where isSet(property) is true.
   * For example, Properties resulting from the use of
   * open or mixed XML content are present if allowed by the Type.
   * the List does not contain duplicates.
   * The order of the Properties in the List begins with getType().getProperties()
   * and the order of the remaining Properties is determined by the implementation.
   * The same list will be returned unless the DataObject is updated so that
   * the contents of the List change.
   * @return the List of Properties currently used in this DataObject.
   */
  List<Property> getInstanceProperties();

  /**
   * Returns the named Property from the current instance properties,
   * or null if not found.  The instance properties are getInstanceProperties().
   * @param propertyName the name of the Property
   * @return the named Property from the DataObject's current instance properties, or null.
   */
  Property getInstanceProperty(String propertyName);

  /**
   * Returns the ChangeSummary with scope covering this dataObject, or null
   * if there is no ChangeSummary.
   * @return the ChangeSummary with scope covering this dataObject, or null.
   */
  ChangeSummary getChangeSummary();

  /**
   * Removes this DataObject from its container, if any.
   * Same as
   *  getContainer().getList(getContainmentProperty()).remove(this) or
   *  getContainer().unset(getContainmentProperty())
   * depending on getContainmentProperty().isMany() respectively.
   */
  void detach();

}