/*
 * Copyright(C) OASIS(R) 2005,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 */
package org.oasisopen.sdo.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.security.AccessController;
import java.security.PrivilegedAction;

import org.oasisopen.sdo.helper.HelperContext;
import org.oasisopen.sdo.helper.HelperContextFactory;

/**
 * This class allows SDO to be configured for a specific environment.  Possible environments
 * include:
 * - Java SE
 * - Application Servers
 * - OSGi environments
 */
public abstract class Environment {

    protected static final String HELPER_CONTEXT_FACTORY = "org.oasisopen.sdo.helper.HelperContextFactory";

    protected static final String ENVIRONMENT = "org.oasisopen.sdo.impl.Environment";

    protected static final String SERVICE_RESOURCE = "META-INF/services/";

    private static Environment INSTANCE;

    public static Environment getInstance() {
        if(null == INSTANCE) {
            ClassLoader classLoader = getContextClassLoader();
            String impl = getImplementationName(ENVIRONMENT);
            if(null == impl) {
                try {
                    impl = getImplementationName(classLoader, ENVIRONMENT);
                    if(null == impl) {
                        throw new CreateEnvironmentException();
                    }
                } catch (IOException e) {
                    throw new CreateEnvironmentException(e);
                }
            }
            try {
                INSTANCE = (Environment) classLoader.loadClass(impl).newInstance();
            } catch(Exception e) {
                throw new CreateEnvironmentException(e);
            }
        }
        return INSTANCE;
    }

    public static void setInstance(Environment environment) {
        INSTANCE = environment;
    }

    protected static HelperContextFactory createHelperContextFactory(ClassLoader classLoader) {
        String impl = getImplementationName(HELPER_CONTEXT_FACTORY);
        if(null == impl) {
            try {
                impl = getImplementationName(classLoader, HELPER_CONTEXT_FACTORY);
                if(null == impl) {
                    throw new CreateHelperContextFactoryException();
                }
            } catch (IOException e) {
                throw new CreateHelperContextFactoryException(e);
            }
        }
        try {
            return (HelperContextFactory) classLoader.loadClass(impl).newInstance();
        } catch(Exception e) {
            throw new CreateHelperContextFactoryException(e);
        }
    }

    /**
     * Get the implementation name from the System property.
     */
    protected static String getImplementationName(final String systemProperty) {
        try {
          return (String)AccessController.doPrivileged(new PrivilegedAction<Object>() {
              public Object run() {
                  return System.getProperty(systemProperty);
              }
          });
        } catch (SecurityException e) {
            return null;
        }
    }

    /**
     * Get the implementation name from "META-INF/services/".
     */
    protected static String getImplementationName(ClassLoader cl, String serviceProvider) throws IOException {
        InputStream is = cl.getResourceAsStream(SERVICE_RESOURCE + serviceProvider);
        if (is == null) return null;
        InputStreamReader in = new InputStreamReader(is, "UTF-8");
        BufferedReader reader = new BufferedReader(in, 128);
        try {
            String line;
            while ((line = reader.readLine()) != null) {
                int i = line.indexOf('#');
                if (i != -1) {
                    line = line.substring(0, i);
                }
                line = line.trim();
                if (line.length() > 0) return line;
            }
            return null;
        } finally {
            reader.close();
        }
    }

    protected static ClassLoader getContextClassLoader() {
        try {
          return (ClassLoader)AccessController.doPrivileged(new PrivilegedAction<Object>() {
              public Object run() {
                  return Thread.currentThread().getContextClassLoader();
              }
          });
        } catch (SecurityException e) {
            return Environment.class.getClassLoader();
        }
    }

    /**
     * <p>Return the default HelperContextFactory.<p>
     * <p>This method is called by the corresponding method on org.oasisopen.sdo.helper.SDO
     * and must have the following behavior:
     * <pre>
     * SDO.getHelperContextFactory() == SDO.getHelperContextFactory();
     * </pre>
     * </p>
     */
    public abstract HelperContextFactory getHelperContextFactory();

    /**
     * <p>Return the HelperContextFactory for the specified implementation.</p>
     * <p>This method is called by the corresponding method on org.oasisopen.sdo.helper.SDO
     * and must have the following behavior:
     * <pre>
     * SDO.getHelperContextFactory(implName) == SDO.getHelperContextFactory(implName);
     * </pre>
     * </p>
     */
    public abstract HelperContextFactory getHelperContextFactory(String implName);

    /**
     * <p>Return the default HelperContext.  The HelperContext returned must have
     * null as the identifier.</p>
     * <p>This method is called by the corresponding method on org.oasisopen.sdo.helper.SDO
     * and must have the following behavior:
     * <pre>
     * SDO.getDefaultHelperContext() == SDO.getDefaultHelperContext();
     * </pre>
     * </p>
     *
     */
    public abstract HelperContext getDefaultHelperContext();

    /**
     * <p>Return the HelperContext with specified identifier, if a HelperContext
     * has been registered with that identifier.
     * <p>This method is called by the corresponding method on org.oasisopen.sdo.helper.SDO
     * and must have the following behavior:
     * <pre>
     * SDO.getHelperContext(identifier) == SDO.getHelperContext(identifier);
     * </pre>
     * </p>
     */
    public abstract HelperContext getHelperContext(String identifier);

    /**
     * Register a HelperContext.
     * @return true if the HelperContext was successfully added (would not be added if a different HelperContext was registered with that identifier.
     */
    public abstract boolean addHelperContext(HelperContext helperContext);

}